

class Buffer:

    """
    Interface for buffers
    """

    def __init__(self, filename=None):
        return

    def open(self, filename):
        return

    def save(self, filename=None):
        return

    def get_modified(self):
        return

    def set_modified(self, value):
        return

    def get_buffer(self):
        return

    def destroy(self):
        return

    def get_title(self):
        return

    def get_mode_interface(self):
        return
